#include <stdio.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <unistd.h>

/*
 * Utility to check the validity of a hostname, or the plausibility of an
 * alleged IP address, via the Name Service Switch.  Result code is zero for
 * a valid/plausible argument, or one otherwise.
 *
 * Normal output is a human readable message; if the identifier is valid, a
 * parenthesised canonical hostname is added in the case of an IP address
 * argument (or a repetition of the IP address if unresolvable), or a valid IP
 * address of the named host for a non-IP-address argument.
 *
 * -i forces output of an IP address only (no output if unresolvable).
 * -n forces output of the canonical hostname only (no output if unresolvable).
 * -q suppresses output - result code only.
 * -I and -N are the same as -i and -n except that dummy output is provided for
 * an unsuccessful result (0.0.0.0 for -I or a hexidecimal derivative of the
 * IP address (00000000 if invalid) for -N).
 */

/* Copyright (C) 2003, 2003 Paul A. Whittaker <Paul.Whittaker@iname.com> */

/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

inline void usage ()

{
    fputs ("Usage: checkhostname [-I|-i|-N|-n|-q] identifier\n", stderr);
    exit (1);
}

int main (int argc, char *argv[])

{
    struct in_addr ip;
    struct hostent *inf = NULL;
    char c, *res;
    unsigned int arg_is_ip = 0, dummy = 0;
    extern int optind;
    enum { normal, ip_only, name_only, quiet } outmode = normal;

    while ((c = getopt (argc, argv, ":iInNq")) != (char) EOF) {
	switch (c) {
	case 'I':
	case 'i':
	    if (outmode)
		usage ();
	    outmode = ip_only;
	    dummy = c == 'I';
	    break;
	case 'N':
	case 'n':
	    if (outmode)
		usage ();
	    outmode = name_only;
	    dummy = c == 'N';
	    break;
	case 'q':
	    if (outmode)
		usage ();
	    outmode = quiet;
	    break;
	case ':':
	case '?':
	    usage ();
	}
    }
    if (optind == argc || *argv[optind] == '-')
	usage ();

    if (inet_aton (argv[optind], &ip)) {
	arg_is_ip = 1;
	inf = gethostbyaddr (&ip, sizeof(ip), AF_INET);
    }

    if (inf != NULL) {
	res = inf -> h_name;
    } else {
	inf = gethostbyname (argv[optind]);
	if (inf != NULL)
	    res = inet_ntoa (*(struct in_addr *)inf -> h_addr_list[0]);
    }

    if (inf == NULL) {
	switch (outmode) {
	case normal:
	    printf ("%s is not a host.\n", argv[optind]);
	    break;
	case ip_only:
	    if (dummy)
		puts ("0.0.0.0");
	    break;
	case name_only:
	    if (dummy)
		puts ("00000000");
	    break;
	case quiet:
	    ;
	}
        return (1);
    } else {
	switch (outmode) {
	case normal:
	    printf ("%s (%s) is a host.\n", argv[optind], res);
	    break;
	case ip_only:
	    puts (arg_is_ip ? argv[optind] : res);
	    break;
	case name_only:
	    if (!arg_is_ip || !inet_aton (inf -> h_name, &ip))
		puts (inf -> h_name);
	    else if (dummy) {
		while (ip.s_addr) {
		    printf ("%02X", ip.s_addr & (in_addr_t) 0xff);
		    ip.s_addr >>= 8;
		}
		putc ('\n', stdout);
	    }
	    break;
	case quiet:
	    ;
	}
    }
    return (0);
}
